/*
 *  BINtegrity aka Syswall
 *
 */

#ifndef _SYSWALL_H_
#define _SYSWALL_H_

#define ORIGIN 1
#define ARGUMENTS 2
#define SYMBOLS 3

#define MOD_REG_RA 16
#define MOD_REG_A1 1
#define MOD_REG_A2 2
#define MOD_REG_A3 4
#define MOD_REG_A4 8
#define MOD_REG_V0 32


struct vetmap_t;

struct vetmap_t {
	unsigned int addr;
	unsigned int syscall_number;
	unsigned int syscall_return;
	unsigned int argv[4];
	unsigned int modreglist;
	unsigned short badlr;
	unsigned short subreturn_valid; 

	unsigned long int ts;
};

struct vm_area_fix_t {
	unsigned int start;
	unsigned int end;
	struct file *f;
	unsigned char dynamic;
};

typedef struct elf_info_t {
	unsigned load_address;
	Elf32_Sym *dynsym;
    uint32_t dynsym_entries;
    char *dynstr;
    Elf32_Half type;
} elf_info;


int vet_syscall(struct file *f, unsigned offset, unsigned syscall_number, unsigned int *regmodlist);
int vet_subreturn(struct file *f, unsigned offset, unsigned *argv, unsigned argc, unsigned int regmodlist, struct vetmap_t **vm);
void disable_page_protection(void);
void enable_page_protection(void);
void do_hook_exit(unsigned hook);

extern int kernel_read(struct file *, loff_t, char *, unsigned long);

// #define DPRINT(...) printk(KERN_INFO __VA_ARGS__);
#define DPRINT(...) ;
#define WARN(...) printk(KERN_WARNING __VA_ARGS__);

#define WARN_BASIC_INFO(void) \
	WARN("Syscall by %s pid %d, tgid %d\n", current->comm, current->pid, current->tgid); \
	WARN("Syscall #%u, argc %u argv %#x, %#x, %#x, %#x\n", syscall_number, argc, argv[0], argv[1], argv[2], argv[3]); \
    WARN("Return address: %#x, subreturn address: %#x\n", return_address, subreturn_address); \
    if (syscall_file) WARN("Return address in %s offset %#x\n", syscall_file->f_path.dentry->d_name.name, syscall_offset); \
	if (library_file) WARN("Subreturn address in %s offset %#x\n", library_file->f_path.dentry->d_name.name, library_offset);

#endif
